@@ -8,9 +8,10 @@ import numpy as np
 from astropy.io import fits
 from astropy.visualization.wcsaxes import WCSAxes
 from astropy.wcs import WCS
+from colorspacious import cspace_converter
 from matplotlib import colormaps as mplcm
 from matplotlib import rc as mplrc
-from matplotlib.colors import ListedColormap, LogNorm, SymLogNorm
+from matplotlib.colors import LinearSegmentedColormap, ListedColormap, LogNorm, SymLogNorm
 from matplotlib.patches import Ellipse
 
 ##############################################################
@@ -91,7 +92,7 @@ def prepare_plot_data(
 
 
 def plot_wcs_data(
-    hdu: Optional[list],
+    hdu: Optional[list] = None,
     fits_path: Optional[str] = None,
     plot_data: Optional[np.ndarray] = None,
     channel: Optional[int] = 0,
@@ -109,6 +110,7 @@ def plot_wcs_data(
     subtract_data: Optional[np.ndarray] = None,
     subtract_channels: Optional[list] = None,
     subtract_overlay_channels: Optional[list] = None,
+    mask_value: Optional[float] = None,
     num_ticks: int = 5,
     log: bool = False,
     symlog: bool = False,
@@ -126,6 +128,8 @@ def plot_wcs_data(
     show: bool = True,
     num_levels: Optional[int] = None,
     interpolation: str = "none",
+    plot_text: Optional[dict] = None,
+    plot_text_color: str = "white",
     **kwargs,
 ) -> None:
     """
@@ -143,6 +147,7 @@ def plot_wcs_data(
     label_font = kwargs.get("label_font", labels)
     tick_font = kwargs.get("tick_font", ticks)
     legend_font = kwargs.get("legend_font", legends)
+    text_font = kwargs.get("text_font", labels)
     # override default figure size
     figsize = kwargs.get("figsize", (14.0, 10.5))
     # override default colormaps
@@ -193,6 +198,10 @@ def plot_wcs_data(
         plot_data = plot_data.copy()
         plot_data -= subtract_data
 
+    # masks an inner region to 0 (helpful for delta Keplerian)
+    if mask_value is not None:
+        plot_data[np.abs(plot_data) <= mask_value] = 0.0
+
     # make a log normalizer
     norm = LogNorm(vmin, vmax) if log else None
     # do a symmetric log normalizer with a linear threshold as a fraction (if < 0) or given value (> 0)
@@ -234,6 +243,8 @@ def plot_wcs_data(
     if overlay_hdu is not None:
         overlay_hdu[0].header["CRVAL1"] = 0.0
         overlay_hdu[0].header["CRVAL2"] = 0.0
+        overlay_hdu[0].header["CRPIX1"] = overlay_hdu[0].header["NAXIS1"] // 2
+        overlay_hdu[0].header["CRPIX2"] = overlay_hdu[0].header["NAXIS2"] // 2
         overlay_wcs = WCS(overlay_hdu[0].header, naxis=2)
 
         if overlay_data is None:
@@ -280,6 +291,17 @@ def plot_wcs_data(
                 alpha=overlay_alpha,
             )
 
+    # add an overlaying text (dictionary [text] -> (x, y))
+    if plot_text is not None:
+        for text in plot_text:
+            plt.text(
+                plot_text[text][0],
+                plot_text[text][1],
+                text,
+                color=plot_text_color,
+                fontsize=text_font,
+            )
+
     # set axis labels
     y_label = r"$\Delta$ DEC"
     x_label = r"$\Delta$ RA"
@@ -451,8 +473,9 @@ def get_wiggle_from_contour(
 
 
 def polar_plot(
-    rs: np.ndarray,
-    phis: np.ndarray,
+    rs: dict,
+    phis: dict,
+    plot_labels: Optional[dict] = {},
     rmax: Optional[float] = None,
     scatter: bool = True,
     rlabel_position: float = 300.0,
@@ -473,6 +496,7 @@ def polar_plot(
     # get font information if given
     tick_font = kwargs.get("tick_font", ticks)
     figsize = kwargs.get("figsize", (12.0, 12.0))
+    legend_font = kwargs.get("legend_font", legends)
     # line attributes
     lines = kwargs.get("lines", linestyles)
     linewidth = kwargs.get("linewidth", lw)
@@ -485,21 +509,63 @@ def polar_plot(
 
     _, ax = plt.subplots(figsize=figsize, subplot_kw={"projection": "polar"})
 
+    if type(rs) != dict:
+        rs = dict(("", rs))
+        phis = dict(("", phis))
+
     if scatter:
-        plt.scatter(
-            phis,
-            rs,
-            s=ps / 50.0,
-            color=colors[0],
-            marker=markers[0],
-            alpha=0.75,
-        )
+        i = 0
+        for var in rs:
+            # adds a random color if we haven't given enough
+            if i > len(color_list) - 1:
+                other_colors = mcolors.CSS4_COLORS
+                rand_color_index = np.random.randint(0, high=len(other_colors))
+                this_color = list(mcolors.CSS4_COLORS.values())[rand_color_index]
+                color_list.append(this_color)
+
+            plt.scatter(
+                phis[var],
+                rs[var],
+                s=ps / 50.0,
+                color=color_list[i],
+                marker=markers[0],
+                alpha=0.75,
+                label=plot_labels[var] if var in plot_labels else None,
+            )
+            i += 1
     else:
-        # split into where the curves are above and below the major axis
-        negative = np.where(phis < 0)
-        positive = np.where(phis > 0)
-        plt.plot(phis[negative], rs[negative], lw=linewidth / 2, c=color_list[0], ls=lines[0])
-        plt.plot(phis[positive], rs[positive], lw=linewidth / 2, c=color_list[0], ls=lines[0])
+        i = 0
+        for var in rs:
+            # adds a random color if we haven't given enough
+            if i > len(color_list) - 1:
+                other_colors = mcolors.CSS4_COLORS
+                rand_color_index = np.random.randint(0, high=len(other_colors))
+                this_color = list(mcolors.CSS4_COLORS.values())[rand_color_index]
+                color_list.append(this_color)
+
+            # split into where the curves are above and below the major axis
+            negative = np.where(phis[var] < 0)
+            positive = np.where(phis[var] > 0)
+            plt.plot(
+                phis[var][negative],
+                rs[var][negative],
+                lw=linewidth / 2,
+                c=color_list[i],
+                ls=lines[0],
+                label=plot_labels[var] if var in plot_labels else None,
+            )
+            plt.plot(
+                phis[var][positive],
+                rs[var][positive],
+                lw=linewidth / 2,
+                c=color_list[i],
+                ls=lines[0],
+            )
+
+            i += 1
+
+    if plot_labels != {}:
+        plt.legend(loc="best", fontsize=legend_font)
 
     if rmax is None:
         ax.set_rlabel_position(rlabel_position)
@@ -621,7 +687,7 @@ def plot_series(
     max_value = -1e30
     for var in xs:
         # adds a random color if we haven't given enough
-        if i > len(color_list):
+        if i > len(color_list) - 1:
             other_colors = mcolors.CSS4_COLORS
             rand_color_index = np.random.randint(0, high=len(other_colors))
             this_color = list(mcolors.CSS4_COLORS.values())[rand_color_index]
@@ -669,7 +735,7 @@ def plot_series(
 
     if len(hlines) > 0:
         for name in hlines:
-            plt.axvline(hlines[name], lw=lw, ls=":", c="k", label=name)
+            plt.axhline(hlines[name], lw=lw, ls=":", c="k", label=name)
 
     if logx:
         plt.xscale("log")
@@ -700,3 +766,36 @@ def plot_series(
         plt.show()
     else:
         plt.close()
+
+
+def hex_to_rgb(value):
+    """Convert hex to RGB values in the range [0, 1]."""
+    value = value.lstrip("#")
+    lv = len(value)
+    return tuple(int(value[i : i + lv // 3], 16) / 255.0 for i in range(0, lv, lv // 3))
+
+
+def create_perceptually_uniform_cmap(start_color: list, end_color: list, N: int = 256):
+    if type(start_color) is str:
+        start_color = (
+            hex_to_rgb(start_color) if "#" in start_color else mcolors.to_rgb(start_color)
+        )
+    if type(end_color) is str:
+        end_color = hex_to_rgb(end_color) if "#" in end_color else mcolors.to_rgb(end_color)
+    # Convert the start and end colors from RGB to LAB color space
+    converter = cspace_converter("sRGB1", "CAM02-UCS")
+    start_color_lab = converter(start_color)
+    end_color_lab = converter(end_color)
+
+    # Create a linear interpolation of colors in LAB color space
+    lab_colors = np.linspace(start_color_lab, end_color_lab, N)
+
+    # Convert the interpolated colors back to RGB
+    converter = cspace_converter("CAM02-UCS", "sRGB1")
+    rgb_colors = converter(lab_colors)
+
+    # Ensure all RGB values are within the valid range [0, 1]
+    rgb_colors = np.clip(rgb_colors, 0, 1)
+
+    # Create and return the colormap
+    return LinearSegmentedColormap.from_list("custom_colormap", rgb_colors)
