@@ -11,6 +11,8 @@ from protomatics.analysis import (
     calc_azimuthal_average,
     calculate_doppler_flip,
     calculate_fourier_amps,
+    get_code_units,
+    get_Q_toomre,
     get_wiggle_amplitude,
     make_ev_dataframe,
     make_grids,
@@ -112,6 +114,10 @@ def test_wiggle_amplitude(fits_name):
         rs, phis, ref_rs=rs, ref_phis=phis, vel_is_zero=False, use_std_as_amp=True
     )
 
+    # calculate with rotation
+    kep_moment1 = calculate_keplerian_moment1(hdu=fits.open(path), rotate=0.785)
+
+    # calculate without rotation
     kep_moment1 = calculate_keplerian_moment1(hdu=fits.open(path))
 
     kep_rs, kep_phis = extract_wiggle(kep_moment1, in_pv_space=False)
@@ -140,6 +146,37 @@ def test_hdf5(hdf5_name):
         path,
         extra_file_keys=["h", "dt", "eta_AD", "Bxyz"],
     )
+    print("Loading with sink/header information")
+    hdf5_df = make_hdf5_dataframe(
+        path,
+        extra_file_keys=["m", "massoftype", "u"],
+    )
+
+    print("Testing Toomre Q")
+    code_units = get_code_units(path, extra_values=["gamma"])
+    udist = code_units["udist"]
+    umass = code_units["umass"]
+    utime = code_units["utime"]
+    code_units["uG"] = udist**3 / (umass * utime**2)
+    code_units["uenergy"] = umass * (udist / utime) ** 2
+    r_annulus = 25.0
+
+    print("Testing Q at (r, phi)")
+    _ = get_Q_toomre(
+        hdf5_df,
+        r_annulus,
+        gamma=code_units["gamma"],
+        code_units=code_units,
+    )
+
+    print("Testing azimuthal average Q at (r)")
+    _ = get_Q_toomre(
+        hdf5_df,
+        r_annulus,
+        gamma=code_units["gamma"],
+        code_units=code_units,
+        az_avg=True,
+    )
 
     print("Testing grid with loaded frame")
     _ = make_interpolated_grid(dataframe=hdf5_df, grid_size=200)
